/*******************************************************************************
 * Tree
 ******************************************************************************/

.readme {
  display:inline-block;
}

.title {
  margin-right: 1em;
  margin-left: 5px;
}

.title + p {
  display: inline;
}

.readme p {
  margin-top: 0pt;
  margin-bottom: 0pt;
}

li {
  list-style-type: none;
  margin-left: -40px;
  padding-left: 20px;
  margin-top: 3px;
  margin-bottom: 3px;
}

li.directory.closed {
  background:url("folder.png") no-repeat left top;
  background-size: 20px 20px;
}

li.directory.open {
  background:url("folder-open.png") no-repeat left top;
  background-size: 20px 20px;
}

li.file {
  background: url("file.png") no-repeat left top;
  background-size: 20px 20px;
}

li.open ul {
  display: block;
}

li.closed ul {
  display: none;
}

/*******************************************************************************
 * Thumbnail                                                                    *
 ******************************************************************************/

.tree li img.thumbnail {
  transition: all .3s ease-out;
  width: 1em;
  height: 1em;
  z-index: 1;
}
.tree li img.thumbnail:hover {
  transform: scale(5);
  transform-origin: center center;
  background-color: white;
  box-shadow: 0 1px 3px rgba(34, 25, 25, 0.4);
}

/*******************************************************************************
 * Tooltips                                                                    *
 ******************************************************************************/

/* Tooltip container */
.tooltip {
  position: relative;
  display: inline-block;
}

/* Tooltip text */
.tooltip .tooltiptext {
  visibility: hidden;
  background-color: black;
  color: #fff;
  text-align: center;
  padding: 5px 0;
  border-radius: 6px;
  width: 5cm;
  top: 100%;
  left: 50%;
  margin-left: -2.5cm; /* Use half of the width (120/2 = 60), to center the tooltip

  /* Position the tooltip text - see examples below! */
  position: absolute;
  z-index: 1;
}

/* Show the tooltip text when you mouse over the tooltip container */
.tooltip:hover .tooltiptext {
  visibility: visible;
}
.tooltip .tooltiptext::after {
  content: " ";
  position: absolute;
  bottom: 100%;  /* At the top of the tooltip */
  left: 50%;
  margin-left: -5px;
  border-width: 5px;
  border-style: solid;
  border-color: transparent transparent black transparent;
}

/* Code */
.log pre {
  overflow: auto;
  height: auto;
  max-height: 30em;
  background:#F8F8FF;
  border:black dashed 1px;
  padding:6px;
}
.log {
  display: none;
}
